@@ -24,7 +24,8 @@ def chain(fixation_XY, text_block, x_thresh=192, y_thresh=32):
     `x_thresh=192`, `y_thresh=32`.
 
     """
-    line_Y = np.array(text_block.line_positions, dtype=int)
+    fixation_XY = np.array(fixation_XY, dtype=int)
+    line_Y = np.array(text_block.midlines, dtype=int)
     dist_X = abs(np.diff(fixation_XY[:, 0]))
     dist_Y = abs(np.diff(fixation_XY[:, 1]))
     end_chain_indices = list(
@@ -56,12 +57,11 @@ def cluster(fixation_XY, text_block):
     """
     try:
         from scipy.cluster.vq import kmeans2
-    except ImportError:
-        raise ImportError(
-            'The cluster method requires SciPy. Run "pip install scipy" to use this method.'
-        )
-    line_Y = np.array(text_block.line_positions, dtype=int)
-    fixation_Y = np.array(fixation_XY[:, 1], dtype=float)
+    except ModuleNotFoundError as e:
+        e.msg = 'The cluster method requires SciPy. Run "pip install scipy" to use this method.'
+        raise
+    fixation_Y = np.array(fixation_XY, dtype=float)[:, 1]
+    line_Y = np.array(text_block.midlines, dtype=int)
     _, clusters = kmeans2(fixation_Y, line_Y, iter=100, minit="matrix", missing="warn")
     return line_Y[clusters]
 
@@ -89,12 +89,18 @@ def merge(fixation_XY, text_block, y_thresh=32, gradient_thresh=0.1, error_thres
     params: `y_thresh=32`, `gradient_thresh=0.1`, `error_thresh=20`.
 
     """
-    line_Y = np.array(text_block.line_positions, dtype=int)
+    fixation_XY = np.array(fixation_XY, dtype=int)
+    line_Y = np.array(text_block.midlines, dtype=int)
     diff_X = np.diff(fixation_XY[:, 0])
     dist_Y = abs(np.diff(fixation_XY[:, 1]))
-    sequence_boundaries = list(
-        np.where(np.logical_or(diff_X < 0, dist_Y > y_thresh))[0] + 1
-    )
+    if text_block.right_to_left:
+        sequence_boundaries = list(
+            np.where(np.logical_or(diff_X > 0, dist_Y > y_thresh))[0] + 1
+        )
+    else:
+        sequence_boundaries = list(
+            np.where(np.logical_or(diff_X < 0, dist_Y > y_thresh))[0] + 1
+        )
     sequence_starts = [0] + sequence_boundaries
     sequence_ends = sequence_boundaries + [len(fixation_XY)]
     sequences = [
@@ -172,11 +178,11 @@ def regress(
     try:
         from scipy.optimize import minimize
         from scipy.stats import norm
-    except ImportError:
-        raise ImportError(
-            'The regress method requires SciPy. Run "pip install scipy" to use this method.'
-        )
-    line_Y = np.array(text_block.line_positions, dtype=int)
+    except ModuleNotFoundError as e:
+        e.msg = 'The regress method requires SciPy. Run "pip install scipy" to use this method.'
+        raise
+    fixation_XY = np.array(fixation_XY, dtype=int)
+    line_Y = np.array(text_block.midlines, dtype=int)
     density = np.zeros((len(fixation_XY), len(line_Y)))
 
     def fit_lines(params):
@@ -217,10 +223,14 @@ def segment(fixation_XY, text_block):
     order.
 
     """
-    line_Y = np.array(text_block.line_positions, dtype=int)
+    fixation_XY = np.array(fixation_XY, dtype=int)
+    line_Y = np.array(text_block.midlines, dtype=int)
     diff_X = np.diff(fixation_XY[:, 0])
     saccades_ordered_by_length = np.argsort(diff_X)
-    line_change_indices = saccades_ordered_by_length[: len(line_Y) - 1]
+    if text_block.right_to_left:
+        line_change_indices = saccades_ordered_by_length[-(len(line_Y) - 1) :]
+    else:
+        line_change_indices = saccades_ordered_by_length[: len(line_Y) - 1]
     current_line_i = 0
     for fixation_i in range(len(fixation_XY)):
         fixation_XY[fixation_i, 1] = line_Y[current_line_i]
@@ -244,14 +254,17 @@ def split(fixation_XY, text_block):
     """
     try:
         from scipy.cluster.vq import kmeans2
-    except ImportError:
-        raise ImportError(
-            'The split method requires SciPy. Run "pip install scipy" to use this method.'
-        )
-    line_Y = np.array(text_block.line_positions, dtype=int)
+    except ModuleNotFoundError as e:
+        e.msg = 'The split method requires SciPy. Run "pip install scipy" to use this method.'
+        raise
+    fixation_XY = np.array(fixation_XY, dtype=int)
+    line_Y = np.array(text_block.midlines, dtype=int)
     diff_X = np.array(np.diff(fixation_XY[:, 0]), dtype=float).reshape(-1, 1)
     centers, clusters = kmeans2(diff_X, 2, iter=100, minit="++", missing="raise")
-    sweep_marker = np.argmin(centers)
+    if text_block.right_to_left:
+        sweep_marker = np.argmax(centers)
+    else:
+        sweep_marker = np.argmin(centers)
     end_line_indices = list(np.where(clusters == sweep_marker)[0] + 1)
     end_line_indices.append(len(fixation_XY))
     start_of_line = 0
@@ -287,13 +300,12 @@ def stretch(
     """
     try:
         from scipy.optimize import minimize
-    except ImportError:
-        raise ImportError(
-            'The stretch method requires SciPy. Run "pip install scipy" to use this method.'
-        )
-    line_Y = np.array(text_block.line_positions, dtype=int)
-    n = len(fixation_XY)
-    fixation_Y = fixation_XY[:, 1]
+    except ModuleNotFoundError as e:
+        e.msg = 'The stretch method requires SciPy. Run "pip install scipy" to use this method.'
+        raise
+    fixation_Y = np.array(fixation_XY, dtype=int)[:, 1]
+    line_Y = np.array(text_block.midlines, dtype=int)
+    n = len(fixation_Y)
     corrected_Y = np.zeros(n)
 
     def fit_lines(params):
@@ -330,7 +342,8 @@ def warp(fixation_XY, text_block):
     order.
 
     """
-    word_XY = np.array(text_block.word_centers, dtype=int)
+    fixation_XY = np.array(fixation_XY, dtype=int)
+    word_XY = np.array(text_block.word_centers(), dtype=int)
     n1 = len(fixation_XY)
     n2 = len(word_XY)
     cost = np.zeros((n1 + 1, n2 + 1))
