@@ -23,7 +23,7 @@ def discard_short_fixations(fixation_sequence, threshold=50):
     """
     if not isinstance(fixation_sequence, _FixationSequence):
         raise TypeError("fixation_sequence should be of type eyekit.FixationSequence")
-    for fixation in fixation_sequence:
+    for fixation in fixation_sequence.iter_without_discards():
         if fixation.duration < threshold:
             fixation.discarded = True
 
@@ -41,8 +41,11 @@ def discard_out_of_bounds_fixations(fixation_sequence, text_block, threshold=100
         raise TypeError("fixation_sequence should be of type eyekit.FixationSequence")
     if not isinstance(text_block, _TextBlock):
         raise TypeError("text_block should be of type eyekit.TextBlock")
+    check_inside_line = threshold > text_block.line_height / 2
     threshold_squared = threshold ** 2
-    for fixation in fixation_sequence:
+    for fixation in fixation_sequence.iter_without_discards():
+        if check_inside_line and text_block.which_line(fixation):
+            continue  # Fixation is inside a line, so skip to next fixation
         for char in text_block:
             distance_squared = (fixation.x - char.x) ** 2 + (fixation.y - char.y) ** 2
             if distance_squared < threshold_squared:
@@ -60,7 +63,10 @@ def snap_to_lines(fixation_sequence, text_block, method="warp", **kwargs):
     not return a copy. Several methods are available, some of which take
     optional parameters or require SciPy to be installed. For a full
     description and evaluation of these methods, see [Carr et al.
-    (2020)](https://osf.io/jg3nc/).
+    (2020)](https://osf.io/jg3nc/). In right-to-left TextBlocks, reading is
+    assumed to be progressing from right to left. In single-line TextBlocks,
+    the `method` parameter has no effect: all fixations will be snapped to the
+    one line.
 
     """
     if not isinstance(fixation_sequence, _FixationSequence):
@@ -73,9 +79,11 @@ def snap_to_lines(fixation_sequence, text_block, method="warp", **kwargs):
         )
     if text_block.n_rows == 1:
         for fixation in fixation_sequence.iter_without_discards():
-            fixation.y = text_block.line_positions[0]
+            fixation.y = text_block.midlines[0]  # move all fixations to midline
     else:
-        fixation_XY = fixation_sequence.XYarray(include_discards=False)
+        fixation_XY = [
+            fixation.xy for fixation in fixation_sequence.iter_without_discards()
+        ]
         corrected_Y = _drift.methods[method](fixation_XY, text_block, **kwargs)
         for fixation, y in zip(fixation_sequence.iter_without_discards(), corrected_Y):
             fixation.y = y
@@ -95,15 +103,17 @@ def align_to_screenshot(
     screenshot_path,
     output_path=None,
     show_text=True,
+    show_guide_lines=True,
     show_bounding_boxes=False,
 ):
     """
 
-    Create an image depicting a PNG screenshot overlaid with a
-    `eyekit.text.TextBlock` in green. If no output path is provided, the
-    output is written to the same directory as the screenshot file. This is
-    useful for establishing the correct `eyekit.text.TextBlock` parameters to
-    match what participants are actually seeing.
+    Given a `eyekit.text.TextBlock` and the path to a PNG screenshot file,
+    produce an image showing the original screenshot overlaid with the text
+    block (shown in green). If no output path is provided, the output image is
+    written to the same directory as the screenshot file. This is useful for
+    establishing `eyekit.text.TextBlock` parameters (position, font size,
+    etc.) that match what participants actually saw in your experiment.
 
     """
     screenshot_path = str(screenshot_path)
@@ -116,26 +126,23 @@ def align_to_screenshot(
     context.set_source_rgb(0.60392, 0.80392, 0.19607)
     context.set_font_face(text_block._font.toy_font_face)
     context.set_font_size(text_block._font.size)
-    context.set_line_width(2)
-    context.move_to(text_block.x_tl, 0)
-    context.line_to(text_block.x_tl, screen_height)
-    context.stroke()
+    if show_guide_lines:
+        context.set_line_width(2)
+        context.move_to(text_block.position[0], 0)
+        context.line_to(text_block.position[0], screen_height)
+        context.stroke()
     for i, line in enumerate(text_block.lines()):
-        if i == 0:
-            context.move_to(0, text_block._first_baseline)
-            context.line_to(screen_width, text_block._first_baseline)
+        if show_guide_lines:
+            context.move_to(0, text_block.baselines[i])
+            context.line_to(screen_width, text_block.baselines[i])
             context.stroke()
             context.set_dash([8, 4])
-        else:
-            context.move_to(text_block.x_tl, line.baseline)
-            context.line_to(screen_width, line.baseline)
-            context.stroke()
         if show_text:
-            context.move_to(line.x_tl, line.baseline)
+            context.move_to(line._x_tl, line.baseline)  # _x_tl is unpadded x_tl
             context.show_text(line.text)
     if show_bounding_boxes:
         context.set_dash([])
-        for word in text_block.words(add_padding=False):
+        for word in text_block.words():
             context.rectangle(*word.box)
             context.stroke()
     if output_path is None:
