@@ -6,9 +6,16 @@ visualizations.
 """
 
 
-from os import path as _path
+from os.path import splitext as _splitext
 import cairocffi as _cairo
-from . import _color, _font
+from .fixation import FixationSequence as _FixationSequence
+from .text import TextBlock as _TextBlock, _Font
+from . import _color
+
+try:
+    from ._version import __version__
+except ImportError:
+    __version__ = "???"
 
 
 _FONT_FACE = "Arial"
@@ -99,6 +106,8 @@ class Image(object):
         the text.
 
         """
+        if not isinstance(text_block, _TextBlock):
+            raise TypeError("text_block should be of type TextBlock")
         if self._text_extents is None:
             self._text_extents = [
                 text_block.x_tl,
@@ -118,9 +127,9 @@ class Image(object):
         rgb_color = _color_to_rgb(color)
         for line in text_block.lines():
             arguments = {
-                "x": line.x_tl,
+                "x": line._x_tl,  # _x_tl is unpadded x_tl
                 "y": line.baseline,
-                "text": line.text,
+                "text": line.display_text,
                 "font": text_block._font,
                 "color": rgb_color,
             }
@@ -135,6 +144,8 @@ class Image(object):
         `color` determines the color of the heatmap.
 
         """
+        if not isinstance(text_block, _TextBlock):
+            raise TypeError("text_block should be of type TextBlock")
         rgb_color = _color_to_rgb(color)
         n = (text_block.n_cols - distribution.shape[1]) + 1
         distribution /= distribution.max()
@@ -142,15 +153,15 @@ class Image(object):
         levels = [subcell_height * i for i in range(n)]
         level = 0
         for ngram in text_block.ngrams(n):
-            r, s, e = ngram.id.split(":")
+            r, s, _ = ngram.location
             if level == n:
                 level = 0
             p = distribution[(int(r), int(s))]
             cell_color = _pseudo_alpha(rgb_color, opacity=p)
             arguments = {
-                "x": ngram.x_tl,
+                "x": ngram._x_tl,
                 "y": ngram.y_tl + subcell_height * level,
-                "width": ngram.width,
+                "width": ngram._x_br - ngram._x_tl,
                 "height": subcell_height,
                 "stroke_width": None,
                 "color": None,
@@ -159,12 +170,6 @@ class Image(object):
             }
             self._add_component(_draw_rectangle, arguments)
             level += 1
-        start_x = text_block.x_tl
-        end_x = text_block.x_br
-        for line_n, line in enumerate(text_block.lines()):
-            if line_n == 0:
-                continue
-            y = line.y_tl
         self.draw_text_block(text_block)
 
     def draw_fixation_sequence(
@@ -175,14 +180,20 @@ class Image(object):
         color="black",
         discard_color="gray",
         number_fixations=False,
+        fixation_radius=None,
     ):
         """
 
-        Draw a `eyekit.fixation.FixationSequence` on the image. Optionally, you can
-        choose whether or not to display saccade lines and discarded fixations, and
-        which colors to use. `number_fixations` is not yet implemented.
+        Draw a `eyekit.fixation.FixationSequence` on the image. Optionally,
+        you can choose whether or not to display saccade lines and discarded
+        fixations, and which colors to use. `number_fixations` is not yet
+        implemented. `fixation_radius` can be used to set a constant radius
+        for all fixations (rather than a radius that is proportional to
+        duration).
 
         """
+        if not isinstance(fixation_sequence, _FixationSequence):
+            raise TypeError("fixation_sequence should be of type FixationSequence")
         rgb_color = _color_to_rgb(color)
         rgb_discard_color = _color_to_rgb(discard_color)
         if show_discards:
@@ -198,11 +209,12 @@ class Image(object):
                 }
                 self._add_component(_draw_line, arguments)
             for fixation in fixation_sequence.iter_with_discards():
+                radius = fixation_radius or _duration_to_radius(fixation.duration)
                 f_color = rgb_discard_color if fixation.discarded else rgb_color
                 arguments = {
                     "x": fixation.x,
                     "y": fixation.y,
-                    "radius": _duration_to_radius(fixation.duration),
+                    "radius": radius,
                     "color": None,
                     "stroke_width": None,
                     "dashed": False,
@@ -223,10 +235,11 @@ class Image(object):
                 }
                 self._add_component(_draw_line, arguments)
             for fixation in fixation_sequence.iter_without_discards():
+                radius = fixation_radius or _duration_to_radius(fixation.duration)
                 arguments = {
                     "x": fixation.x,
                     "y": fixation.y,
-                    "radius": _duration_to_radius(fixation.duration),
+                    "radius": radius,
                     "color": None,
                     "stroke_width": None,
                     "dashed": False,
@@ -240,6 +253,7 @@ class Image(object):
         fixation_sequence,
         color_match="black",
         color_mismatch="red",
+        fixation_radius=None,
     ):
         """
 
@@ -249,6 +263,10 @@ class Image(object):
         comparing the outputs of two different drift correction algorithms.
 
         """
+        if not isinstance(fixation_sequence, _FixationSequence):
+            raise TypeError("fixation_sequence should be of type FixationSequence")
+        if not isinstance(reference_sequence, _FixationSequence):
+            raise TypeError("reference_sequence should be of type FixationSequence")
         rgb_color_match = _color_to_rgb(color_match)
         rgb_color_mismatch = _color_to_rgb(color_mismatch)
         path = [fixation.xy for fixation in fixation_sequence.iter_with_discards()]
@@ -267,10 +285,11 @@ class Image(object):
                 color = rgb_color_match
             else:
                 color = rgb_color_mismatch
+            radius = fixation_radius or _duration_to_radius(fixation.duration)
             arguments = {
                 "x": fixation.x,
                 "y": fixation.y,
-                "radius": _duration_to_radius(fixation.duration),
+                "radius": radius,
                 "color": None,
                 "stroke_width": None,
                 "dashed": False,
@@ -289,7 +308,7 @@ class Image(object):
         rgb_color = _color_to_rgb(color)
         arguments = {
             "path": [start_xy, end_xy],
-            "stroke_width": stroke_width,
+            "stroke_width": float(stroke_width),
             "color": rgb_color,
             "dashed": dashed,
         }
@@ -308,11 +327,11 @@ class Image(object):
         rgb_color = _color_to_rgb(color) if color else None
         rgb_fill_color = _color_to_rgb(fill_color) if fill_color else None
         arguments = {
-            "x": x,
-            "y": y,
-            "radius": radius,
+            "x": float(x),
+            "y": float(y),
+            "radius": float(radius),
             "color": rgb_color,
-            "stroke_width": stroke_width,
+            "stroke_width": float(stroke_width),
             "dashed": dashed,
             "fill_color": rgb_fill_color,
         }
@@ -344,12 +363,12 @@ class Image(object):
         else:
             x = rect
         arguments = {
-            "x": x,
-            "y": y,
-            "width": width,
-            "height": height,
+            "x": float(x),
+            "y": float(y),
+            "width": float(width),
+            "height": float(height),
             "color": rgb_color,
-            "stroke_width": stroke_width,
+            "stroke_width": float(stroke_width),
             "dashed": dashed,
             "fill_color": rgb_fill_color,
         }
@@ -368,12 +387,12 @@ class Image(object):
             font_face = _FONT_FACE
         if font_size is None:
             font_size = _FONT_SIZE
-        font = _font.Font(font_face, font_size)
+        font = _Font(font_face, font_size)
         rgb_color = _color_to_rgb(color)
         arguments = {
-            "x": x,
-            "y": y,
-            "text": text,
+            "x": float(x),
+            "y": float(y),
+            "text": str(text),
             "font": font,
             "color": rgb_color,
             "annotation": True,
@@ -393,7 +412,7 @@ class Image(object):
 
         """
         output_path = str(output_path)
-        image_format = _path.splitext(output_path)[1][1:].upper()
+        image_format = _splitext(output_path)[1][1:].upper()
         if image_format not in ["PDF", "EPS", "SVG", "PNG"]:
             raise ValueError(
                 "Unrecognized format. Use .pdf, .eps, or .svg for vector output, or .png for raster output."
@@ -461,6 +480,9 @@ class Image(object):
                 ) * scale + crop_margin * 2
             if image_format == "PDF":
                 surface = _cairo.PDFSurface(output_path, image_width, image_height)
+                surface.set_metadata(
+                    _cairo.PDF_METADATA_CREATOR, f"eyekit {__version__}"
+                )
             elif image_format == "EPS":
                 surface = _cairo.PSSurface(output_path, image_width, image_height)
                 surface.set_eps(True)
@@ -617,7 +639,7 @@ class Figure(object):
 
         """
         output_path = str(output_path)
-        figure_format = _path.splitext(output_path)[1][1:].upper()
+        figure_format = _splitext(output_path)[1][1:].upper()
         if figure_format not in ["PDF", "EPS", "SVG"]:
             raise ValueError("Unrecognized format. Use .pdf, .eps, or .svg.")
         figure_width = _mm_to_pts(width)
@@ -661,6 +683,7 @@ class Figure(object):
         """
         if figure_format == "PDF":
             surface = _cairo.PDFSurface(output_path, figure_width, figure_height)
+            surface.set_metadata(_cairo.PDF_METADATA_CREATOR, f"eyekit {__version__}")
         elif figure_format == "EPS":
             surface = _cairo.PSSurface(output_path, figure_width, figure_height)
             surface.set_eps(True)
@@ -713,7 +736,7 @@ class Figure(object):
         v_padding = _mm_to_pts(self._v_padding)
         h_padding = _mm_to_pts(self._h_padding)
         e_padding = _mm_to_pts(self._e_padding)
-        letter_font = _font.Font(self._letter_font_face, self._letter_font_size)
+        letter_font = _Font(self._letter_font_face, self._letter_font_size)
         y = e_padding
         for row in self._grid:
             x = e_padding
@@ -754,7 +777,7 @@ class Figure(object):
                     components.append((_draw_text, arguments))
                     caption_advance += letter_font.calculate_width(letter_prefix)
                 if image._caption:
-                    caption_font = _font.Font(
+                    caption_font = _Font(
                         image._caption_font_face, image._caption_font_size
                     )
                     arguments = {
@@ -861,6 +884,8 @@ class Booklet(object):
         Add a `Figure` to a new page in the booklet.
 
         """
+        if not isinstance(figure, Figure):
+            raise TypeError("figure should be of type Figure.")
         self._figures.append(figure)
 
     def save(self, output_path, width=210, height=297):
@@ -872,11 +897,12 @@ class Booklet(object):
 
         """
         output_path = str(output_path)
-        if _path.splitext(output_path)[1][1:].lower() != "pdf":
+        if _splitext(output_path)[1][1:].lower() != "pdf":
             raise ValueError("Books must be saved in PDF format.")
         page_width = _mm_to_pts(width)
         page_height = _mm_to_pts(height)
         surface = _cairo.PDFSurface(output_path, page_width, page_height)
+        surface.set_metadata(_cairo.PDF_METADATA_CREATOR, f"eyekit {__version__}")
         context = _cairo.Context(surface)
         for figure in self._figures:
             figure._render_to_booklet(surface, context, page_width)
@@ -901,7 +927,8 @@ def _draw_line(context, scale, path, color, stroke_width, dashed):
 
 
 def _draw_circle(context, scale, x, y, radius, color, stroke_width, dashed, fill_color):
-    context.arc(x, y, radius, 0, 6.283185307179586)
+    context.new_sub_path()  # prevent initial line segment
+    context.arc(x, y, radius, 0, 6.3)
     if color and stroke_width:
         context.set_source_rgb(*color)
         context.set_line_width(stroke_width / scale)
